@@ -80,6 +80,14 @@ corHMM<-function(phy, data, rate.cat, rate.mat=NULL, node.states=c("joint", "mar
 		rate[is.na(rate)]=max(rate, na.rm=TRUE)+1
 		model.set.final$rate <- rate
 		model.set.final$index.matrix <- rate.mat
+        ## for precursor type models ##
+        col.sums <- which(colSums(rate.mat, na.rm=TRUE) == 0)
+        row.sums <- which(rowSums(rate.mat, na.rm=TRUE) == 0)
+        drop.states <- col.sums[which(col.sums == row.sums)]
+        if(length(drop.states > 0)){
+            model.set.final$liks[,drop.states] <- 0
+        }
+        ###############################
 	}
 
 	lower = rep(lb, model.set.final$np)
@@ -513,11 +521,11 @@ dev.corhmm <- function(p,phy,liks,Q,rate,root.p) {
 	#Obtain an object of all the unique ancestors
 	anc <- unique(phy$edge[,1])
 	k.rates <- dim(Q)[2] / 2
-	
 	if (any(is.nan(p)) || any(is.infinite(p))) return(1000000)
-	
+    
 	Q[] <- c(p, 0)[rate]
 	diag(Q) <- -rowSums(Q)
+	
 	for (i  in seq(from = 1, length.out = nb.node)) {
 		#the ancestral node at row i is called focal
 		focal <- anc[i]
